'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var Core, comparer;

comparer = function(v, k, b, is_array, path, r) {
  var new_path, ref, ref1;
  new_path = path.concat([k]);
  if (is_array && !(((v != null ? v.id : void 0) && (v != null ? v.id : void 0) === ((ref = b[k]) != null ? ref.id : void 0)) || ((v != null ? v._id : void 0) && (v != null ? v._id : void 0) === ((ref1 = b[k]) != null ? ref1._id : void 0))) || !((v != null) && ((b != null ? b[k] : void 0) != null) && (v instanceof Object))) {
    return r.push(new_path.concat([v]));
  }
  return r.push.apply(r, Core.diff(v, b[k], new_path));
};

var Core$1 = Core = {
  context: null,
  queues: {
    immediate: {
      tasks: [],
      nextIndex: 0,
      nextHandle: 1
    },
    deferred: {
      tasks: [],
      nextIndex: 0,
      nextHandle: 1
    }
  },
  clock: 0,
  queueTask: function(task, defer) {
    var q;
    if (defer) {
      q = Core.queues.deferred;
      if (!q.tasks.length) {
        Promise.resolve().then(function() {
          Core.processUpdates(q);
          return Core.clock++;
        });
      }
      q.tasks.push(task);
      return q.nextHandle++;
    }
    Core.queues.immediate.tasks.push(task);
    return Core.queues.immediate.nextHandle++;
  },
  cancelTask: function(handle, defer) {
    var index, q;
    q = defer ? Core.queues.deferred : Core.queues.immediate;
    index = handle - (q.nextHandle - q.tasks.length);
    if (q.nextIndex <= index) {
      return q.tasks[index] = null;
    }
  },
  processUpdates: function(q) {
    var count, err, mark, task;
    count = 0;
    mark = 0;
    while (q.nextIndex < q.tasks.length) {
      if (!(task = q.tasks[q.nextIndex])) {
        q.nextIndex++;
        continue;
      }
      if (q.nextIndex > mark) {
        if (count++ > 5000) {
          console.error('Exceeded max task recursion');
          q.nextIndex = 0;
          return q.tasks = [];
        }
        mark = q.tasks.length;
      }
      try {
        task(task.value);
        task.handle = null;
        task.value = null;
      } catch (error) {
        err = error;
        console.error(err);
      }
      q.nextIndex++;
    }
    q.nextIndex = 0;
    q.tasks = [];
  },
  run: function(fn) {
    var execute;
    execute = !Core.queues.immediate.tasks.length;
    fn();
    if (execute) {
      return Core.processUpdates(Core.queues.immediate);
    }
  },
  setContext: function(newContext, fn) {
    var context, ret;
    context = Core.context;
    Core.context = newContext;
    ret = fn();
    Core.context = context;
    return ret;
  },
  root: function(fn) {
    var d;
    return Core.setContext({
      disposables: d = []
    }, function() {
      return fn(function() {
        var disposable, i, len;
        for (i = 0, len = d.length; i < len; i++) {
          disposable = d[i];
          disposable();
        }
        d = [];
      });
    });
  },
  ignore: function(fn) {
    var disposables;
    ({disposables} = Core.context || {});
    return Core.setContext({disposables}, fn);
  },
  isObject: function(obj) {
    var ref;
    return obj !== null && ((ref = typeof obj) === 'object' || ref === 'function');
  },
  isFunction: function(val) {
    return typeof val === 'function';
  },
  diff: function(a, b, path = []) {
    var i, k, l, len, r, v;
    r = [];
    if (!Core.isObject(a) || (b == null)) {
      if (a !== b) {
        r.push(path.concat([a]));
      }
    } else if (Array.isArray(a)) {
      for (k = i = 0, len = a.length; i < len; k = ++i) {
        v = a[k];
        if ((b != null ? b[k] : void 0) !== v) {
          comparer(v, k, b, true, path, r);
        }
      }
      if ((b != null ? b.length : void 0) > a.length) {
        l = a.length;
        while (l < b.length) {
          r.push(path.concat([l, void 0]));
          l++;
        }
      }
    } else {
      for (k in a) {
        v = a[k];
        if ((b != null ? b[k] : void 0) !== v) {
          comparer(v, k, b, false, path, r);
        }
      }
      for (k in b) {
        v = b[k];
        if (!(k in a)) {
          r.push(path.concat([k, void 0]));
        }
      }
    }
    return r;
  },
  clone: function(v) {
    var k, obj;
    if (!Core.isObject(v)) {
      return v;
    }
    if (Array.isArray(v)) {
      return v.slice(0);
    }
    obj = {};
    for (k in v) {
      obj[k] = v[k];
    }
    return obj;
  },
  unwrap: function(item, deep) {
    var k, result, unwrapped, v;
    if (result = item != null ? item._state : void 0) {
      return result;
    }
    if (!(deep && Core.isObject(item) && !Core.isFunction(item) && !(item instanceof Element) && !(item instanceof DocumentFragment))) {
      return item;
    }
    for (k in item) {
      v = item[k];
      if ((unwrapped = Core.unwrap(v, true)) !== v) {
        item[k] = unwrapped;
      }
    }
    return item;
  }
};

var Sync;

var Sync$1 = Sync = class Sync {
  constructor(handler, {defer = true} = {}) {
    var ref;
    this.handler = handler;
    this.execute = this.execute.bind(this);
    this.execute.defer = defer;
    this.context = {
      fn: this.execute,
      disposables: []
    };
    this.execute();
    if ((ref = Core$1.context) != null) {
      ref.disposables.push(this.dispose.bind(this));
    }
  }

  execute() {
    if (this.__disposed) {
      return;
    }
    this.clean();
    return Core$1.setContext(this.context, this.handler);
  }

  clean() {
    var disposable, i, len, ref;
    ref = this.context.disposables;
    for (i = 0, len = ref.length; i < len; i++) {
      disposable = ref[i];
      disposable();
    }
    return this.context.disposables = [];
  }

  dispose() {
    if (this.__disposed) {
      return;
    }
    this.clean();
    return this.__disposed = true;
  }

};

function symbolObservablePonyfill(root) {
	var result;
	var Symbol = root.Symbol;

	if (typeof Symbol === 'function') {
		if (Symbol.observable) {
			result = Symbol.observable;
		} else {
			result = Symbol('observable');
			Symbol.observable = result;
		}
	} else {
		result = '@@observable';
	}

	return result;
}

/* global window */

var root;

if (typeof self !== 'undefined') {
  root = self;
} else if (typeof window !== 'undefined') {
  root = window;
} else if (typeof global !== 'undefined') {
  root = global;
} else if (typeof module !== 'undefined') {
  root = module;
} else {
  root = Function('return this')();
}

var result = symbolObservablePonyfill(root);

var Signal, counter;

counter = 0;

var Signal$1 = Signal = class Signal {
  constructor(value) {
    this.sid = 's_' + counter++;
    this.__subscriptions = new Set();
    this.__value = value;
    Object.defineProperty(this, 'value', {
      get: function() {
        var ref;
        if ((ref = Core$1.context) != null ? ref.fn : void 0) {
          Core$1.context.disposables.push(this.subscribe(Core$1.context.fn).unsubscribe);
        }
        return this.__value;
      },
      configurable: true
    });
    Object.defineProperty(this, result, {
      get: function() {
        return this;
      }
    });
  }

  peek() {
    return this.__value;
  }

  next(value, notify = true) {
    this.__value = value;
    if (notify) {
      return this.notify('next', value);
    }
  }

  subscribe(observer) {
    if (typeof observer !== 'object' || observer === null) {
      observer = {
        next: observer,
        error: arguments[1],
        complete: arguments[2]
      };
    }
    this.__subscriptions.add(observer);
    return {
      unsubscribe: this.unsubscribe.bind(this, observer)
    };
  }

  unsubscribe(observer) {
    return this.__subscriptions.delete(observer);
  }

  notify(type, value) {
    if (!this.__subscriptions.size) {
      return;
    }
    return Core$1.run(() => {
      var handler, ref, sub;
      ref = this.__subscriptions;
      for (sub of ref) {
        if (!(handler = sub[type])) {
          continue;
        }
        if (handler.handle != null) {
          Core$1.cancelTask(sub.handle, sub.defer);
        }
        handler.value = value;
        handler.handle = Core$1.queueTask(handler, handler.defer);
      }
    });
  }

};

var Selector, sequenceCounter;

sequenceCounter = 0;

var Selector$1 = Selector = class Selector extends Signal$1 {
  constructor(handler, {defer} = {}) {
    var ref;
    super();
    this.handler = handler;
    Object.defineProperty(this, 'value', {
      get: function() {
        var ref;
        if ((ref = Core$1.context) != null ? ref.fn : void 0) {
          Core$1.context.disposables.push(this.subscribe(Core$1.context.fn).unsubscribe);
        }
        if (!this.context.disposables.length) {
          this.execute();
        }
        return this.__value;
      },
      configurable: false,
      writeable: false
    });
    this.execute = this.execute.bind(this);
    this.execute.defer = defer;
    this.context = {
      fn: this.execute,
      disposables: [],
      pure: true
    };
    if ((ref = Core$1.context) != null) {
      ref.disposables.push(this.dispose.bind(this));
    }
  }

  execute() {
    var context, currentSeq, is_sleeping, new_val;
    if (this.__disposed) {
      return;
    }
    this.seq = currentSeq = sequenceCounter++;
    this.clean();
    is_sleeping = !this.__subscriptions.size;
    context = is_sleeping ? null : this.context;
    new_val = Core$1.setContext(context, () => {
      return this.handler(this.__value);
    });
    if (new_val && Core$1.isObject(new_val)) {
      if ('then' in new_val) {
        new_val.then(function(new_val) {
          if (this.__disposed || this.seq !== currentSeq) {
            return;
          }
          return this.resolve(new_val);
        }).catch(function(err) {
          return this.notify('error', err);
        });
        return;
      }
      // TODO check if observable
      if (!('_state' in new_val) && 'subscribe' in new_val) {
        this.__subscribable = new_val.subscribe((val) => {
          return this.resolve(val);
        }, (err) => {
          return this.notify('error', err);
        }, () => {
          var ref;
          return (ref = this.__subscribable) != null ? ref.unsubscribe() : void 0;
        });
        return;
      }
    }
    return this.resolve(new_val);
  }

  resolve(new_val) {
    new_val = Core$1.unwrap(new_val, true);
    if (new_val === this.__value && !Core$1.isObject(new_val)) {
      return;
    }
    return super.next(new_val, !this._skipNotify);
  }

  next() {}

  peek() {
    if (!this.context.disposables.length) {
      this.execute();
    }
    return super.peek();
  }

  subscribe(fn) {
    var disposable, old_size;
    old_size = this.__subscriptions.size;
    disposable = super.subscribe(fn);
    if (!old_size && this.__subscriptions.size && !this.context.disposables.length) {
      this._skipNotify = true;
      this.execute();
      delete this._skipNotify;
    }
    return disposable;
  }

  unsubscribe(fn) {
    super.unsubscribe(fn);
    return Core$1.queueTask(() => {
      if (!this.__subscriptions.size) {
        return this.clean();
      }
    }, true);
  }

  clean() {
    var disposable, i, len, ref, ref1;
    ref = this.context.disposables;
    for (i = 0, len = ref.length; i < len; i++) {
      disposable = ref[i];
      disposable();
    }
    this.context.disposables = [];
    if ((ref1 = this.__subscribable) != null) {
      ref1.unsubscribe();
    }
    return this.__subscribable = null;
  }

  dispose() {
    if (this.__disposed) {
      return;
    }
    this.notify('complete');
    this.clean();
    return this.__disposed = true;
  }

};

// Shamelessly lifted straight from SurplusJS https://github.com/adamhaile/surplus
var NOINSERT, NOMATCH, RECONCILE_ARRAY_BATCH, RECONCILE_ARRAY_BITS, RECONCILE_ARRAY_INC, RECONCILE_ARRAY_MASK, equable, findGreatestIndexLEQ, insertOrAppend, longestPositiveIncreasingSubsequence, reconcileArrays;

NOMATCH = -1;

NOINSERT = -2;

RECONCILE_ARRAY_BATCH = 0;

RECONCILE_ARRAY_BITS = 16;

RECONCILE_ARRAY_INC = 1 << RECONCILE_ARRAY_BITS;

RECONCILE_ARRAY_MASK = RECONCILE_ARRAY_INC - 1;

// reconcile the content of parent from ns to us
// see ivi's excellent writeup of diffing arrays in a vdom library: 
// https://github.com/ivijs/ivi/blob/2c81ead934b9128e092cc2a5ef2d3cabc73cb5dd/packages/ivi/src/vdom/implementation.ts#L1187
// this code isn't identical, since we're diffing real dom nodes to nodes-or-strings, 
// but the core methodology of trimming ends and reversals, matching nodes, then using
// the longest increasing subsequence to minimize DOM ops is inspired by ivi.
var reconcileArrays$1 = reconcileArrays = function(parent, ns, us, multiple) {
  var doloop, i, j, k, lcs, lcsj, len1, m, n, next, nmax, nmin, ntext, ntextk, nx, preserved, src, u, ul, ulen, umax, umin, utext, utexti, ux;
  ulen = us.length;
  nmin = 0;
  nmax = ns.length - 1;
  umin = 0;
  umax = ulen - 1;
  n = ns[nmin];
  u = us[umin];
  nx = ns[nmax];
  ux = us[umax];
  ul = nx.nextSibling;
  i = void 0;
  j = void 0;
  k = void 0;
  doloop = true;
  // scan over common prefixes, suffixes, and simple reversals
  while (doloop) {
    doloop = false;
    next = false;
    // common prefix, u === n
    while (equable(u, n, umin, us)) {
      umin++;
      nmin++;
      if (umin > umax || nmin > nmax) {
        next = true;
        break;
      }
      u = us[umin];
      n = ns[nmin];
    }
    if (next) {
      continue;
    }
    // common suffix, ux === nx
    while (equable(ux, nx, umax, us)) {
      ul = nx;
      umax--;
      nmax--;
      if (umin > umax || nmin > nmax) {
        next = true;
        break;
      }
      ux = us[umax];
      nx = ns[nmax];
    }
    if (next) {
      continue;
    }
    // reversal u === nx, have to swap node forward
    while (equable(u, nx, umin, us)) {
      doloop = true;
      parent.insertBefore(nx, n);
      umin++;
      nmax--;
      if (umin > umax || nmin > nmax) {
        next = true;
        break;
      }
      u = us[umin];
      nx = ns[nmax];
    }
    if (next) {
      continue;
    }
    // reversal ux === n, have to swap node back
    while (equable(ux, n, umax, us)) {
      doloop = true;
      if (ul === null) {
        parent.appendChild(n);
      } else {
        parent.insertBefore(n, ul);
      }
      ul = n;
      umax--;
      nmin++;
      if (umin > umax || nmin > nmax) {
        next = true;
        break;
      }
      ux = us[umax];
      n = ns[nmin];
    }
    if (next) {
      continue;
    }
  }
  // if that covered all updates, just need to remove any remaining nodes and we're done
  if (umin > umax) {
    // remove any remaining nodes
    while (nmin <= nmax) {
      parent.removeChild(ns[nmax]);
      nmax--;
    }
    return;
  }
  // if that covered all current nodes, just need to insert any remaining updates and we're done
  if (nmin > nmax) {
    // insert any remaining nodes
    while (umin <= umax) {
      insertOrAppend(parent, us[umin], ul, umin, us);
      umin++;
    }
    return;
  }
  // simple cases don't apply, have to actually match up nodes and figure out minimum DOM ops
  // loop through nodes and mark them with a special property indicating their order
  // we'll then go through the updates and look for those properties
  // in case any of the updates have order properties left over from earlier runs, we 
  // use the low bits of the order prop to record a batch identifier.
  // I'd much rather use a Map than a special property, but Maps of objects are really
  // slow currently, like only 100k get/set ops / second
  // for Text nodes, all that matters is their order, as they're easily, interchangeable
  // so we record their positions in ntext[]
  ntext = [];
  // update global batch identifer
  RECONCILE_ARRAY_BATCH = (RECONCILE_ARRAY_BATCH + 1) % RECONCILE_ARRAY_INC;
  i = nmin;
  j = (nmin << RECONCILE_ARRAY_BITS) + RECONCILE_ARRAY_BATCH;
  while (i <= nmax) {
    n = ns[i];
    // add or update special order property
    if (n.__special_order === void 0) {
      Object.defineProperty(n, '__special_order', {
        value: j,
        writable: true
      });
    } else {
      n.__special_order = j;
    }
    if (n instanceof Text) {
      ntext.push(i);
    }
    i++;
    j += RECONCILE_ARRAY_INC;
  }
  // now loop through us, looking for the order property, otherwise recording NOMATCH
  src = new Array(umax - umin + 1);
  utext = [];
  preserved = 0;
  i = umin;
  while (i <= umax) {
    u = us[i];
    if (typeof u === 'string') {
      utext.push(i);
      src[i - umin] = NOMATCH;
    } else if ((j = u.__special_order) !== void 0 && (j & RECONCILE_ARRAY_MASK) === RECONCILE_ARRAY_BATCH) {
      j >>= RECONCILE_ARRAY_BITS;
      src[i - umin] = j;
      ns[j] = null;
      preserved++;
    } else {
      src[i - umin] = NOMATCH;
    }
    i++;
  }
  if (preserved === 0 && nmin === 0 && nmax === ns.length - 1) {
    // no nodes preserved, use fast clear and append
    if (multiple) {
      while (umin <= umax) {
        insertOrAppend(parent, us[umin], ns[0], umin, us);
        umin++;
      }
      for (m = 0, len1 = ns.length; m < len1; m++) {
        n = ns[m];
        parent.removeChild(n);
      }
      return;
    }
    parent.textContent = '';
    while (umin <= umax) {
      insertOrAppend(parent, us[umin], null, umin, us);
      umin++;
    }
    return;
  }
  // find longest common sequence between ns and us, represented as the indices 
  // of the longest increasing subsequence in src
  lcs = longestPositiveIncreasingSubsequence(src);
  // we know we can preserve their order, so march them as NOINSERT
  i = 0;
  while (i < lcs.length) {
    src[lcs[i]] = NOINSERT;
    i++;
  }
  /*
        0   1   2   3   4   5   6   7
  ns    = [ n,  n,  t,  n,  n,  n,  t,  n ]
            |          /   /       /
            |        /   /       /
            +------/---/-------/----+
                 /   /       /      |
  us    = [ n,  s,  n,  n,  s,  n,  s,  n ]
  src   = [-1, -1,  4,  5, -1,  7, -1,  1 ]
  lis   = [         2,  3,      5]
                j
  utext = [     1,          4,      6 ]
            i
  ntext = [         2,              6 ]
                k
   */
  // replace strings in us with Text nodes, reusing Text nodes from ns when we can do so without moving them
  utexti = 0;
  lcsj = 0;
  ntextk = 0;
  i = 0;
  j = 0;
  k = 0;
  while (i < utext.length) {
    utexti = utext[i];
    // need to answer qeustion "if utext[i] falls between two lcs nodes, is there an ntext between them which we can reuse?"
    // first, find j such that lcs[j] is the first lcs node *after* utext[i]
    while (j < lcs.length && (lcsj = lcs[j]) < utexti - umin) {
      j++;
    }
    // now, find k such that ntext[k] is the first ntext *after* lcs[j-1] (or after start, if j === 0)
    while (k < ntext.length && (ntextk = ntext[k], j !== 0) && ntextk < src[lcs[j - 1]]) {
      k++;
    }
    // if ntext[k] < lcs[j], then we know ntext[k] falls between lcs[j-1] (or start) and lcs[j] (or end)
    // that means we can re-use it without moving it
    if (k < ntext.length && (j === lcs.length || ntextk < src[lcsj])) {
      n = ns[ntextk];
      u = us[utexti];
      if (n.data !== u) {
        n.data = u;
      }
      ns[ntextk] = null;
      us[utexti] = n;
      src[utexti] = NOINSERT;
      k++;
    } else {
      // if we didn't find one to re-use, make a new Text node
      us[utexti] = document.createTextNode(us[utexti]);
    }
    i++;
  }
  // remove stale nodes in ns
  while (nmin <= nmax) {
    n = ns[nmin];
    if (n !== null) {
      parent.removeChild(n);
    }
    nmin++;
  }
  // insert new nodes
  while (umin <= umax) {
    ux = us[umax];
    if (src[umax - umin] !== NOINSERT) {
      if (ul === null) {
        parent.appendChild(ux);
      } else {
        parent.insertBefore(ux, ul);
      }
    }
    ul = ux;
    umax--;
  }
};

insertOrAppend = function(parent, node, marker, i, us) {
  if (typeof node === 'string') {
    node = us[i] = document.createTextNode(node);
  }
  if (marker === null) {
    parent.appendChild(node);
  } else {
    parent.insertBefore(node, marker);
  }
};

equable = function(u, n, i, us) {
  if (u === n) {
    return true;
  } else if (typeof u === 'string' && n instanceof Text) {
    if (n.data !== u) {
      n.data = u;
    }
    us[i] = n;
    return true;
  } else {
    return false;
  }
};

// return an array of the indices of ns that comprise the longest increasing subsequence within ns
longestPositiveIncreasingSubsequence = function(ns) {
  var i, iss, j, l, len, n, pre, seq;
  seq = [];
  iss = [];
  l = -1;
  pre = new Array(ns.length);
  i = 0;
  len = ns.length;
  while (i < len) {
    n = ns[i];
    if (n < 0) {
      i++;
      continue;
    }
    j = findGreatestIndexLEQ(seq, n);
    if (j !== -1) {
      pre[i] = iss[j];
    }
    if (j === l) {
      l++;
      seq[l] = n;
      iss[l] = i;
    } else if (n < seq[j + 1]) {
      seq[j + 1] = n;
      iss[j + 1] = i;
    }
    i++;
  }
  i = iss[l];
  while (l >= 0) {
    seq[l] = i;
    i = pre[i];
    l--;
  }
  return seq;
};

findGreatestIndexLEQ = function(seq, n) {
  var hi, lo, mid;
  // invariant: lo is guaranteed to be index of a value <= n, hi to be >
  // therefore, they actually start out of range: (-1, last + 1)
  lo = -1;
  hi = seq.length;
  // fast path for simple increasing sequences
  if (hi > 0 && seq[hi - 1] <= n) {
    return hi - 1;
  }
  while (hi - lo > 1) {
    mid = Math.floor((lo + hi) / 2);
    if (seq[mid] > n) {
      hi = mid;
    } else {
      lo = mid;
    }
  }
  return lo;
};

var mapSelector, multipleExpressions, normalizeIncomingArray, singleExpression;

normalizeIncomingArray = function(normalized, array) {
  var i, item, len;
  i = 0;
  len = array.length;
  while (i < len) {
    item = array[i];
    if (item instanceof Node) {
      normalized.push(item);
    } else if (item === null || item === true || item === false) {

    // matches null, undefined, true or false
    // skip
    } else if (Array.isArray(item)) {
      normalizeIncomingArray(normalized, item);
    } else if (typeof item === 'string') {
      normalized.push(item);
    } else {
      normalized.push(item.toString());
    }
    i++;
  }
  return normalized;
};

singleExpression = function(parent, accessor) {
  var current;
  current = null;
  return new Sync$1(function() {
    var array, child, l, len1, len2, m, t, value;
    if ((value = accessor()) === current) {
      return;
    }
    t = typeof value;
    if (t === 'string') {
      if (current) {
        return current = parent.firstChild.data = value;
      }
      return current = parent.textContent = value;
    } else if ('number' === t || 'boolean' === t || value instanceof Date || value instanceof RegExp) {
      value = value.toString();
      if (current) {
        return current = parent.firstChild.data = value;
      }
      return current = parent.textContent = value;
    } else if ((value == null) || t === 'boolean') {
      return current = parent.textContent = '';
    } else if (value instanceof Node) {
      if (Array.isArray(current)) {
        if (current.length === 0) {
          parent.appendChild(value);
        } else if (current.length === 1) {
          parent.replaceChild(value, current[0]);
        } else {
          parent.textContent = '';
          parent.appendChild(value);
        }
      } else if (current === '' || (current == null)) {
        parent.appendChild(value);
      } else {
        parent.replaceChild(value, parent.firstChild);
      }
      return current = value;
    } else if (Array.isArray(value)) {
      array = normalizeIncomingArray([], value);
      if (array.length === 0) {
        parent.textContent = '';
      } else {
        if (Array.isArray(current)) {
          if (current.length === 0) {
            for (l = 0, len1 = array.length; l < len1; l++) {
              child = array[l];
              parent.appendChild(child);
            }
          } else {
            reconcileArrays$1(parent, current, array);
          }
        } else if (!current) {
          for (m = 0, len2 = array.length; m < len2; m++) {
            child = array[m];
            parent.appendChild(child);
          }
        } else {
          reconcileArrays$1(parent, [parent.firstChild], array);
        }
      }
      return current = array;
    } else if ('sid' in value) {
      return singleExpression(parent, function() {
        return value.value;
      });
    } else {
      throw new Error("content must be Node, stringable, or array of same");
    }
  });
};

multipleExpressions = function(parent, accessor) {
  var nodes;
  nodes = [];
  return new Sync$1(function() {
    var array, child, i, l, len1, marker, node, ref, ref1, ref2, t, value;
    marker = null;
    value = accessor();
    t = typeof value;
    parent = ((ref = nodes[0]) != null ? ref.parentNode : void 0) || parent;
    if (t === 'string' || 'number' === t || 'boolean' === t || value instanceof Date || value instanceof RegExp) {
      if (((ref1 = nodes[0]) != null ? ref1.nodeType : void 0) === 3) {
        nodes[0].data = value.toString();
        marker = nodes[0];
      } else {
        value = document.createTextNode(value.toString());
        if (nodes[0]) {
          parent.replaceChild(value, nodes[0]);
        } else {
          parent.appendChild(value);
        }
        nodes[0] = marker = value;
      }
    } else if (value instanceof Node) {
      if (nodes[0]) {
        if (nodes[0] !== value) {
          parent.replaceChild(value, nodes[0]);
        }
      } else {
        parent.appendChild(value);
      }
      nodes[0] = marker = value;
    } else if (Array.isArray(value)) {
      array = normalizeIncomingArray([], value);
      if (array.length) {
        if (!nodes.length) {
          for (i = l = 0, len1 = array.length; l < len1; i = ++l) {
            child = array[i];
            parent.appendChild(child);
            nodes[i] = child;
          }
          marker = nodes[i - 1];
        } else {
          reconcileArrays$1(parent, nodes, array, true);
          nodes = array;
          marker = nodes[nodes.length - 1];
        }
      }
    } else if ((value != null) && 'sid' in value) {
      return multipleExpressions(parent, function() {
        return value.value;
      });
    }
    // handle nulls
    if (marker == null) {
      if (nodes[0] === parent.firstChild && nodes.length > 1 && nodes[nodes.length - 1] === parent.lastChild) {
        parent.textContent = '';
        value = document.createTextNode('');
        parent.appendChild(value);
        marker = nodes[0] = value;
      } else if (((ref2 = nodes[0]) != null ? ref2.nodeType : void 0) === 3) {
        nodes[0].data = '';
        marker = nodes[0];
      } else {
        value = document.createTextNode('');
        if (nodes[0]) {
          parent.replaceChild(value, nodes[0]);
        } else {
          parent.appendChild(value);
        }
        marker = nodes[0] = value;
      }
    }
    // trim extras
    while (marker !== (node = nodes[nodes.length - 1])) {
      parent.removeChild(node);
      nodes.length = nodes.length - 1;
    }
  });
};

mapSelector = function(valueAccessor, mapFn) {
  var disposables, length, list, mapped;
  mapped = [];
  list = [];
  disposables = [];
  length = 0;
  Core$1.context.disposables.push(function() {
    var d, l, len1;
    for (l = 0, len1 = disposables.length; l < len1; l++) {
      d = disposables[l];
      d();
    }
    return disposables = [];
  });
  return new Selector$1(function() {
    var d, end, i, indexedItems, item, itemIndex, j, l, len1, len2, len3, m, n, newEnd, newLength, newList, newListUnwrapped, newMapped, start, tempDisposables;
    newList = valueAccessor();
    // non-arrays
    newListUnwrapped = (newList != null ? newList._state : void 0) || newList;
    if (!Array.isArray(newListUnwrapped)) {
      if ((newListUnwrapped == null) || newListUnwrapped === false) {
        mapped = [];
        list = [];
        for (l = 0, len1 = disposables.length; l < len1; l++) {
          d = disposables[l];
          d();
        }
        disposables = [];
        return;
      }
      if (list[0] === newListUnwrapped) {
        return mapped[0];
      }
      for (m = 0, len2 = disposables.length; m < len2; m++) {
        d = disposables[m];
        d();
      }
      disposables = [];
      list[0] = newListUnwrapped;
      return mapped[0] = Core$1.root(function(dispose) {
        disposables[0] = dispose;
        return mapFn(newList);
      });
    }
    newLength = newListUnwrapped.length;
    if (newLength === 0) {
      if (length !== 0) {
        list = [];
        mapped = [];
        length = 0;
        for (n = 0, len3 = disposables.length; n < len3; n++) {
          d = disposables[n];
          d();
        }
        disposables = [];
      }
    } else if (length === 0) {
      i = 0;
      while (i < newLength) {
        list[i] = newListUnwrapped[i];
        mapped[i] = Core$1.root(function(dispose) {
          disposables[i] = dispose;
          return mapFn((typeof newList.peek === "function" ? newList.peek(i) : void 0) || newList[i], i);
        });
        i++;
      }
      length = newLength;
    } else {
      newMapped = new Array(newLength);
      tempDisposables = new Array(newLength);
      indexedItems = new Map();
      end = Math.min(length, newLength);
      // reduce from both ends
      start = 0;
      while (start < end && newListUnwrapped[start] === list[start]) {
        start++;
      }
      end = length - 1;
      newEnd = newLength - 1;
      while (end >= 0 && newEnd >= 0 && newListUnwrapped[newEnd] === list[end]) {
        newMapped[newEnd] = mapped[end];
        tempDisposables[newEnd] = disposables[newEnd];
        end--;
        newEnd--;
      }
      // create indices
      j = newEnd;
      while (j >= start) {
        item = newListUnwrapped[j];
        itemIndex = indexedItems.get(item);
        if (itemIndex != null) {
          itemIndex.push(j);
        } else {
          indexedItems.set(item, [j]);
        }
        j--;
      }
      // find old items
      i = start;
      while (i <= end) {
        item = list[i];
        itemIndex = indexedItems.get(item);
        if ((itemIndex != null) && itemIndex.length > 0) {
          j = itemIndex.pop();
          newMapped[j] = mapped[i];
          tempDisposables[j] = disposables[i];
        } else {
          disposables[i]();
        }
        i++;
      }
      // set all new values
      j = start;
      while (j < newLength) {
        if (newMapped.hasOwnProperty(j)) {
          mapped[j] = newMapped[j];
          disposables[i] = tempDisposables[i];
        } else {
          mapped[j] = Core$1.root(function(dispose) {
            disposables[j] = dispose;
            return mapFn((typeof newList.peek === "function" ? newList.peek(j) : void 0) || newList[j], j);
          });
        }
        j++;
      }
      // truncate extra length
      length = mapped.length = disposables.length = newLength;
      // save list for next iteration
      list = newListUnwrapped.slice(0);
    }
    return mapped;
  });
};

var Renderer = {
  assign: function(a, b) {
    var k;
    for (k in b) {
      a[k] = b[k];
    }
  },
  wrap: function(accessor, fn) {
    return new Sync$1(function() {
      return fn(Core$1.unwrap(accessor()));
    });
  },
  insert: function(parent, multiple, accessor) {
    if (multiple) {
      multipleExpressions(parent, accessor);
    } else {
      singleExpression(parent, accessor);
    }
  },
  map: mapSelector
};

var DEFINED, Handler;

DEFINED = {
  '_state': true,
  'on': true,
  'peek': true
};

var Handler$1 = Handler = class Handler {
  constructor(_state) {
    var i, len, method, ref;
    this._state = _state;
    this.__subscriptions = new Set();
    this._child_subscriptions = {};
    ref = Object.keys(DEFINED).slice(2);
    for (i = 0, len = ref.length; i < len; i++) {
      method = ref[i];
      this[method] = this[method].bind(this);
    }
  }

  get(target, property) {
    var ref, value;
    if (DEFINED[property]) {
      return this[property];
    }
    if (!((ref = Core$1.context) != null ? ref.fn : void 0) || property === 'length' || typeof property === 'symbol') {
      return target[property];
    }
    if ((value = target[property]) != null) {
      if (Core$1.isFunction(value)) {
        return value;
      }
      if (Core$1.isObject(value) && Object.isFrozen(value)) {
        value = Core$1.clone(value);
        target[property] = value;
      }
      value = Handler.wrap(value);
    }
    Core$1.context.disposables.push(this.on(property, Core$1.context.fn).unsubscribe);
    return value;
  }

  set(target, property, value) {
    return true;
  }

  deleteProperty(target, property) {
    return true;
  }

  subscribe(fn) {
    this.__subscriptions.add(fn);
    return {
      unsubscribe: () => {
        return this.__subscriptions.delete(fn);
      }
    };
  }

  notify(value) {
    if (!this.__subscriptions.size) {
      return;
    }
    return Core$1.run(() => {
      var ref, sub;
      ref = this.__subscriptions;
      for (sub of ref) {
        if (sub.handle != null) {
          Core$1.cancelTask(sub.handle, sub.defer);
        }
        sub.value = value;
        sub.handle = Core$1.queueTask(sub, sub.defer);
      }
    });
  }

};

Handler.wrappers = new WeakMap();

Handler.wrap = function(value) {
  var handler, proxy, wrapper;
  if (!(Core$1.isObject(value) && !(value instanceof Element))) {
    return value;
  }
  if (!(wrapper = Handler.wrappers.get(value))) {
    proxy = new Proxy(value, handler = new Handler(value));
    Handler.wrappers.set(value, {proxy, handler});
  } else {
    ({proxy} = wrapper);
  }
  return proxy;
};

Handler.handler = function(value) {
  var handler, proxy, wrapper;
  if (!(Core$1.isObject(value) && !(value instanceof Element))) {
    return value;
  }
  if (!(wrapper = Handler.wrappers.get(value))) {
    proxy = new Proxy(value, handler = new Handler(value));
    Handler.wrappers.set(value, {proxy, handler});
  } else {
    ({handler} = wrapper);
  }
  return handler;
};

var methods;

methods = {
  trigger: function(property, value, notify) {
    var subs;
    subs = this._child_subscriptions[property];
    if (subs != null ? subs.size : void 0) {
      Core$1.run(function() {
        var sub;
        for (sub of subs) {
          if (sub.handle != null) {
            Core$1.cancelTask(sub.handle, sub.defer);
          }
          sub.value = value;
          sub.handle = Core$1.queueTask(sub, sub.defer);
        }
      });
    }
    if (notify) {
      return this.notify(this._state);
    }
  },
  peek: function(property) {
    var value;
    value = this._state[property];
    if (Core$1.isObject(value) && Object.isFrozen(value)) {
      value = Core$1.clone(value);
      this._state[property] = value;
    }
    return Handler$1.wrap(value);
  },
  on: function(property, fn) {
    var base, disposable, handler, value;
    value = this._state[property];
    disposable = null;
    if (Core$1.isObject(value) && !(Core$1.isFunction(value) || value instanceof Element)) {
      handler = Handler$1.handler(value);
      disposable = handler.subscribe(fn);
    }
    (base = this._child_subscriptions)[property] || (base[property] = new Set());
    this._child_subscriptions[property].add(fn);
    return {
      unsubscribe: () => {
        if (disposable) {
          disposable.unsubscribe();
        }
        return this._child_subscriptions[property].delete(fn);
      }
    };
  }
};

Object.assign(Handler$1.prototype, methods);

var methods$1 = methods;

var State, setNested;

setNested = function(item, changes) {
  var handler, isArray, notify, property, value;
  handler = Handler$1.handler(item);
  isArray = Array.isArray(item);
  if (arguments.length === 3) {
    notify = isArray || !(arguments[1] in item);
    if (arguments[2] === void 0) {
      delete item[arguments[1]];
      if (isArray) {
        item.length--;
      }
    } else {
      item[arguments[1]] = arguments[2];
    }
    handler.trigger(arguments[1], arguments[2], notify);
    return;
  }
  for (property in changes) {
    value = changes[property];
    notify = isArray || !(property in item);
    if (value === void 0) {
      delete item[property];
    } else {
      item[property] = value;
    }
    handler.trigger(property, value, notify);
  }
};

var index = State = class State {
  constructor(_state) {
    var k, ref;
    this._state = _state;
    this._disposables = [];
    this._selectors = [];
    this._child_subscriptions = {};
    for (k in this._state) {
      this._defineProperty(k);
    }
    if ((ref = Core$1.context) != null) {
      ref.disposables.push(this.dispose.bind(this));
    }
  }

  set() {
    var changes, current, i, property, ref, ref1, temp, value;
    if ((ref = Core$1.context) != null ? ref.pure : void 0) {
      return console.log('Cannot update in a Selector');
    }
    if (arguments.length === 1) {
      ref1 = arguments[0];
      for (property in ref1) {
        value = ref1[property];
        this._setProperty(property, value);
      }
      return;
    }
    current = this._state;
    changes = arguments[arguments.length - 1];
    i = 0;
    while (i < arguments.length - 1 && ((temp = current[arguments[i]]) != null)) {
      current = temp;
      i++;
    }
    setNested(current, changes);
  }

  replace() {
    var change, current, i, j, len, property, ref, temp, value;
    if (arguments.length === 1) {
      if (!(arguments[0] instanceof Object)) {
        return console.log('replace must be provided a replacement state');
      }
      ref = Core$1.diff(arguments[0], this._state);
      for (j = 0, len = ref.length; j < len; j++) {
        change = ref[j];
        this.replace(change);
      }
      return;
    }
    if (arguments.length === 2) {
      this._setProperty(arguments[0], arguments[1]);
      return;
    }
    current = this._state;
    value = arguments[arguments.length - 1];
    property = arguments[arguments.length - 2];
    i = 0;
    while (i < arguments.length - 2 && ((temp = current[arguments[i]]) != null)) {
      current = temp;
      i++;
    }
    setNested(current, property, value);
  }

  select(...selections) {
    var j, key, len, selection, selector;
    for (j = 0, len = selections.length; j < len; j++) {
      selection = selections[j];
      if (Core$1.isFunction(selection) || 'subscribe' in selection) {
        selector = Core$1.isFunction(selection) ? new Selector$1(selection) : selection;
        this._selectors.push(selector);
        this._disposables.push(selector.subscribe((value) => {
          var change, key, l, len1, ref, val;
          ref = [].concat(...((function() {
            var ref, results;
            ref = value || {};
            results = [];
            for (key in ref) {
              val = ref[key];
              results.push(Core$1.diff(val, this._state[key], [key]));
            }
            return results;
          }).call(this)));
          for (l = 0, len1 = ref.length; l < len1; l++) {
            change = ref[l];
            this.replace(...change);
          }
        }));
        continue;
      }
      for (key in selection) {
        selector = selection[key];
        ((key, selector) => {
          if (!(key in this)) {
            this._defineProperty(key);
          }
          selector = Core$1.isFunction(selector) ? new Selector$1(selector) : selector;
          this._selectors.push(selector);
          return this._disposables.push(selector.subscribe((value) => {
            var change, l, len1, ref;
            ref = Core$1.diff(value, this._state[key], [key]);
            for (l = 0, len1 = ref.length; l < len1; l++) {
              change = ref[l];
              this.replace(...change);
            }
          }));
        })(key, selector);
      }
    }
  }

  dispose() {
    var disposable, j, len, ref;
    ref = this._disposables;
    for (j = 0, len = ref.length; j < len; j++) {
      disposable = ref[j];
      disposable.unsubscribe();
    }
    delete this._selectors;
  }

  _setProperty(property, value) {
    if (!(property in this)) {
      this._defineProperty(property);
    }
    if (value === void 0) {
      delete this._state[property];
    } else {
      this._state[property] = value;
    }
    return this.trigger(property, value);
  }

  _defineProperty(property) {
    return Object.defineProperty(this, property, {
      get: () => {
        var ref, value;
        if (!((ref = Core$1.context) != null ? ref.fn : void 0)) {
          return this._state[property];
        }
        if ((value = this._state[property]) != null) {
          if (Core$1.isFunction(value)) {
            return value;
          }
          if (Core$1.isObject(value) && Object.isFrozen(value)) {
            value = Core$1.clone(value);
            this._state[property] = value;
          }
          value = Handler$1.wrap(value);
        }
        Core$1.context.disposables.push(this.on(property, Core$1.context.fn).unsubscribe);
        return value;
      }
    });
  }

};

Object.assign(State.prototype, methods$1);

var DEFINED$1, ImmutableHandler;

DEFINED$1 = {
  '_state': true,
  'peek': true,
  'map': true
};

var ImmutableHandler$1 = ImmutableHandler = class ImmutableHandler {
  constructor(target, _state, root) {
    this._state = _state;
    this.root = root;
    this.peek = this.peek.bind(this, target);
    if (Array.isArray(this._state)) {
      this.map = this.map.bind(this, target);
    }
    this.stateClock = target.clock || Core$1.clock;
  }

  get(target, property) {
    var base, base1, ref, value;
    if (this.stateClock < target.clock) {
      this.resolveState(target);
    }
    if (property === 'length' || typeof property === 'symbol') {
      return this._state[property];
    }
    if (property.endsWith('$')) {
      property = property.slice(0, -1);
      if (Core$1.context.fn) {
        target[property] || (target[property] = {
          _subs: new Set(),
          clock: Core$1.clock
        });
        (base = target[property])._subPath || (base._subPath = new Set());
        target[property]._subPath.add(Core$1.context.fn);
        Core$1.context.disposables.push(target[property]._subPath.delete(Core$1.context.fn));
      }
      return this._state[property];
    }
    if (DEFINED$1[property]) {
      return this[property];
    }
    if (!((ref = Core$1.context) != null ? ref.fn : void 0)) {
      return this._state[property];
    }
    if (((value = this._state[property]) != null) && Core$1.isObject(value) && !(value instanceof Element)) {
      if (Core$1.isFunction(value)) {
        return value;
      }
      target[property] || (target[property] = {
        clock: Core$1.clock
      });
      (base1 = target[property]).path || (base1.path = target.path.concat([property]));
      value = new Proxy(target[property], new ImmutableHandler(target[property], value, this.root));
    }
    Core$1.context.disposables.push(this.subscribe(target, property, Core$1.context.fn).unsubscribe);
    return value;
  }

  set(target, property, value) {
    return true;
  }

  deleteProperty(target, property) {
    return true;
  }

  subscribe(target, property, fn) {
    var base;
    target[property] || (target[property] = {
      clock: Core$1.clock
    });
    (base = target[property])._subs || (base._subs = new Set());
    target[property]._subs.add(fn);
    return {
      unsubscribe: function() {
        return target[property]._subs.delete(fn);
      }
    };
  }

  peek(target, property) {
    var base, value;
    if (this.stateClock < target.clock) {
      this.resolveState(target);
    }
    value = this._state[property];
    if ((value == null) || Core$1.isFunction(value) || !Core$1.isObject(value) || (value instanceof Element)) {
      return value;
    }
    target[property] || (target[property] = {});
    (base = target[property]).path || (base.path = target.path.concat([property]));
    return new Proxy(target[property], new ImmutableHandler(target[property], value, this.root));
  }

  map(target, fn) {
    if (this.stateClock < target.clock) {
      this.resolveState(target);
    }
    return this._state.map((value, i, a) => {
      var base;
      if (!((value == null) || Core$1.isFunction(value) || !Core$1.isObject(value) || (value instanceof Element))) {
        target[i] || (target[i] = {});
        (base = target[i]).path || (base.path = target.path.concat([i]));
        value = new Proxy(target[i], new ImmutableHandler(target[i], value, this.root));
      }
      return fn(value, i, a);
    });
  }

  resolveState(target) {
    var current, i, l;
    current = this.root._state;
    i = 0;
    l = target.path.length;
    while (i < l) {
      if (current) {
        current = current[target.path[i]];
      }
      i++;
    }
    this._state = current;
    return this.stateClock = Core$1.clock;
  }

};

var ImmutableState,
  indexOf = [].indexOf;

var index$1 = ImmutableState = class ImmutableState {
  constructor(_state) {
    var k, ref;
    this._state = _state;
    this._disposables = [];
    this._selectors = [];
    this._subscriptions = {};
    for (k in this._state) {
      this._defineProperty(k);
    }
    if ((ref = Core$1.context) != null) {
      ref.disposables.push(this.dispose.bind(this));
    }
  }

  set() {
    var changes, j, k, len, notify, path, property, ref, ref1, ref2, state, subKeys, subPaths, subs, value;
    if ((ref = Core$1.context) != null ? ref.pure : void 0) {
      return console.log('Cannot update in a Selector');
    }
    if (arguments.length === 1) {
      ref1 = arguments[0];
      for (property in ref1) {
        value = ref1[property];
        this._setProperty(property, value);
      }
      return;
    }
    changes = arguments[arguments.length - 1];
    ({state, subs, subPaths} = this._resolvePath(arguments, arguments.length - 1));
    if (!state) {
      return;
    }
    notify = Array.isArray(state);
    for (property in changes) {
      value = changes[property];
      notify = notify || !(indexOf.call(state, property) >= 0);
      if (value === void 0) {
        delete state[property];
      } else {
        state[property] = value;
      }
      if (subs) {
        this.trigger(subs != null ? (ref2 = subs[property]) != null ? ref2._subs : void 0 : void 0, value);
      }
    }
    if (notify) {
      this.trigger(subs != null ? subs._subs : void 0, state);
    }
    for (j = 0, len = subPaths.length; j < len; j++) {
      path = subPaths[j];
      this.trigger(path.subs, path.state);
    }
    // prune subs
    if (value === void 0) {
      delete subs[property];
    } else if (Array.isArray(value) && !value.length) {
      subKeys = ['_subs', 'clock', '_subPath'];
      for (k in subs[property]) {
        if (!(indexOf.call(subKeys, k) >= 0)) {
          delete subs[property][k];
        }
      }
    }
  }

  replace() {
    var change, j, l, len, len1, notify, path, property, ref, ref1, state, subPaths, subs, value;
    if (arguments.length === 1) {
      if (!(arguments[0] instanceof Object)) {
        return console.log('replace must be provided a replacement state');
      }
      ref = Core$1.diff(arguments[0], this._state);
      for (j = 0, len = ref.length; j < len; j++) {
        change = ref[j];
        this.replace(change);
      }
      return;
    }
    if (arguments.length === 2) {
      this._setProperty(arguments[0], arguments[1]);
      return;
    }
    value = arguments[arguments.length - 1];
    property = arguments[arguments.length - 2];
    ({state, subs, subPaths} = this._resolvePath(arguments, arguments.length - 2));
    if (!state) {
      return;
    }
    if (value === void 0 && Array.isArray(state)) {
      state.length -= 1;
      notify = true;
    } else {
      notify = Array.isArray(state) || !(indexOf.call(state, property) >= 0);
      if (value === void 0) {
        delete state[property];
      } else {
        state[property] = value;
      }
    }
    if (subs) {
      this.trigger((ref1 = subs[property]) != null ? ref1._subs : void 0, value);
    }
    if (notify) {
      this.trigger(subs._subs, state);
    }
    for (l = 0, len1 = subPaths.length; l < len1; l++) {
      path = subPaths[l];
      this.trigger(path.subs, path.state);
    }
  }

  select(...selections) {
    var j, key, len, selection, selector;
    for (j = 0, len = selections.length; j < len; j++) {
      selection = selections[j];
      if (Core$1.isFunction(selection) || 'subscribe' in selection) {
        selector = Core$1.isFunction(selection) ? new Selector$1(selection) : selection;
        this._selectors.push(selector);
        this._disposables.push(selector.subscribe((value) => {
          var change, key, l, len1, ref, val;
          ref = [].concat(...((function() {
            var ref, results;
            ref = value || {};
            results = [];
            for (key in ref) {
              val = ref[key];
              results.push(Core$1.diff(val, this._state[key], [key]));
            }
            return results;
          }).call(this)));
          for (l = 0, len1 = ref.length; l < len1; l++) {
            change = ref[l];
            this.replace(...change);
          }
        }));
        continue;
      }
      for (key in selection) {
        selector = selection[key];
        ((key, selector) => {
          if (!(key in this)) {
            this._defineProperty(key);
          }
          selector = Core$1.isFunction(selector) ? new Selector$1(selector) : selector;
          this._selectors.push(selector);
          return this._disposables.push(selector.subscribe((value) => {
            var change, l, len1, ref;
            ref = Core$1.diff(value, this._state[key], [key]);
            for (l = 0, len1 = ref.length; l < len1; l++) {
              change = ref[l];
              this.replace(...change);
            }
          }));
        })(key, selector);
      }
    }
  }

  on(property, fn) {
    var base, base1;
    (base = this._subscriptions)[property] || (base[property] = {
      clock: Core$1.clock
    });
    (base1 = this._subscriptions[property])._subs || (base1._subs = new Set());
    this._subscriptions[property]._subs.add(fn);
    return {
      unsubscribe: () => {
        return this._subscriptions[property]._subs.delete(fn);
      }
    };
  }

  trigger(subs, value) {
    if (!(subs != null ? subs.size : void 0)) {
      return;
    }
    return Core$1.run(function() {
      var sub;
      for (sub of subs) {
        if (sub.handle != null) {
          Core$1.cancelTask(sub.handle, sub.defer);
        }
        sub.value = value;
        sub.handle = Core$1.queueTask(sub, sub.defer);
      }
    });
  }

  dispose() {
    var disposable, j, len, ref;
    ref = this._disposables;
    for (j = 0, len = ref.length; j < len; j++) {
      disposable = ref[j];
      disposable.unsubscribe();
    }
    delete this._selectors;
  }

  _resolvePath(path, length) {
    var currentState, currentSubs, i, ref, subPaths;
    currentState = this._state;
    currentSubs = this._subscriptions;
    subPaths = [];
    i = 0;
    while (i < length) {
      if (!currentSubs[path[i]]) {
        currentSubs[path[i]] = {};
      }
      currentSubs = currentSubs[path[i]];
      if (currentState != null) {
        if (currentSubs.clock !== Core$1.clock) {
          currentState[path[i]] = Core$1.clone(currentState[path[i]]);
          currentSubs.clock = Core$1.clock;
        }
        currentState = currentState[path[i]];
      }
      if (currentSubs != null ? (ref = currentSubs._subPath) != null ? ref.size : void 0 : void 0) {
        subPaths.push({
          state: currentState,
          subs: currentSubs._subPath
        });
      }
      i++;
    }
    return {
      state: currentState,
      subs: currentSubs,
      subPaths
    };
  }

  _setProperty(property, value) {
    var k, ref, ref1, subKeys;
    if (!(property in this)) {
      this._defineProperty(property);
    }
    if (value === void 0) {
      delete this._state[property];
    } else {
      this._state[property] = value;
    }
    this.trigger((ref = this._subscriptions[property]) != null ? ref._subs : void 0, value);
    this.trigger((ref1 = this._subscriptions[property]) != null ? ref1._subPath : void 0, value);
    // prune subs
    if (value === void 0) {
      delete this._subscriptions[property];
    } else if (Array.isArray(value) && !value.length) {
      subKeys = ['_subs', 'clock', '_subPath'];
      for (k in this._subscriptions[property]) {
        if (!(indexOf.call(subKeys, k) >= 0)) {
          delete this._subscriptions[property][k];
        }
      }
    }
  }

  _defineProperty(property) {
    Object.defineProperty(this, property, {
      get: () => {
        var base, base1, ref, value;
        if (!((ref = Core$1.context) != null ? ref.fn : void 0)) {
          return this._state[property];
        }
        if (((value = this._state[property]) != null) && Core$1.isObject(value) && !(value instanceof Element)) {
          if (Core$1.isFunction(value)) {
            return value;
          }
          (base = this._subscriptions)[property] || (base[property] = {
            clock: Core$1.clock
          });
          (base1 = this._subscriptions[property]).path || (base1.path = [property]);
          value = new Proxy(this._subscriptions[property], new ImmutableHandler$1(this._subscriptions[property], value, this));
        }
        Core$1.context.disposables.push(this.on(property, Core$1.context.fn).unsubscribe);
        return value;
      }
    });
    return Object.defineProperty(this, property + '$', {
      get: () => {
        var base, base1, fn;
        if (fn = Core$1.context.fn) {
          (base = this._subscriptions)[property] || (base[property] = {
            _subs: new Set(),
            clock: Core$1.clock
          });
          (base1 = this._subscriptions[property])._subPath || (base1._subPath = new Set());
          this._subscriptions[property]._subPath.add(Core$1.context.fn);
          Core$1.context.disposables.push(() => {
            return this._subscriptions[property]._subPath.delete(fn);
          });
        }
        return this._state[property];
      }
    });
  }

};

var index$2 = Object.assign({
  ignore: Core$1.ignore.bind(Core$1),
  root: Core$1.root.bind(Core$1),
  unwrap: Core$1.unwrap.bind(Core$1)
}, Renderer);

exports.default = index$2;
exports.State = index;
exports.ImmutableState = index$1;
exports.Signal = Signal$1;
exports.Selector = Selector$1;
exports.Sync = Sync$1;
